import typing


class Labels:
    __slots__ = ('_map', '_key')

    def __init__(self, map: typing.Mapping[str, str]) -> None:
        self._map = map
        self._key = '{{{0}}}'.format(','.join([
            '{0}="{1}"'.format(
                k, v.replace('\\', r'\\').replace('\n', r'\n').replace('"', r'\"')
            )
            for k, v in sorted(map.items())
        ]))

    @property
    def key(self) -> str:
        """Metric labels as {...} string to use as suffix for the metric name in prometheus output"""
        return self._key

    @property
    def map(self) -> typing.Mapping[str, str]:
        """Original mapping key got constructed from"""
        return self._map


LabelsData = Labels | typing.Mapping[str, str]


class Path:
    __slots__ = ('_name', '_labels')

    def __init__(
        self,
        name: str,
        labels: typing.Mapping[str, str] = {},
    ) -> None:
        self._name = name
        self._labels = Labels(labels)

    @property
    def name(self) -> str:
        """Metric name"""
        return self._name

    @property
    def labels(self) -> Labels:
        """Metric labels"""
        return self._labels
